/////////////////////////////////////////////////////////////////////////////
//    WinMerge:  an interactive diff/merge utility
//    Copyright (C) 1997-2000  Thingamahoochie Software
//    Author: Dean Grimm
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
/////////////////////////////////////////////////////////////////////////////
// DiffView.cpp : implementation of the CDiffView class
//

#include "stdafx.h"
#include "Merge.h"

#include "MainFrm.h"
#include "MergeDoc.h"
#include "DiffView.h"
#include "textfile.h"
#include "MainFrm.h"
#include "resource.h"
#include "EditFile.h"
#include "DirDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


#define OFFSET_FIRST	2
#define OFFSET_OTHER	6

/////////////////////////////////////////////////////////////////////////////
// CDiffView

IMPLEMENT_DYNCREATE(CDiffView, CListViewEx)

BEGIN_MESSAGE_MAP(CDiffView, CListViewEx)
	//{{AFX_MSG_MAP(CDiffView)
	ON_COMMAND(ID_FILE_SAVE, OnFileSave)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE, OnUpdateFileSave)
	ON_WM_VSCROLL()
	ON_WM_LBUTTONDOWN()
	ON_WM_KEYUP()
	ON_WM_KEYDOWN()
	ON_WM_RBUTTONDOWN()
	ON_WM_CONTEXTMENU()
	ON_COMMAND(ID_POPUP_SAVE, OnPopupSave)
	ON_UPDATE_COMMAND_UI(ID_POPUP_SAVE, OnUpdatePopupSave)
	ON_COMMAND(ID_L2R, OnL2R)
	ON_UPDATE_COMMAND_UI(ID_L2R, OnUpdateL2R)
	ON_COMMAND(ID_R2L, OnR2L)
	ON_UPDATE_COMMAND_UI(ID_R2L, OnUpdateR2L)
	ON_COMMAND(ID_UNDO, OnUndo)
	ON_UPDATE_COMMAND_UI(ID_UNDO, OnUpdateUndo)
	ON_COMMAND(ID_NEXTDIFF, OnNextdiff)
	ON_UPDATE_COMMAND_UI(ID_NEXTDIFF, OnUpdateNextdiff)
	ON_COMMAND(ID_PREVDIFF, OnPrevdiff)
	ON_UPDATE_COMMAND_UI(ID_PREVDIFF, OnUpdatePrevdiff)
	ON_UPDATE_COMMAND_UI(ID_POPUP_EDITFILE, OnUpdateEditFile)
	ON_COMMAND(ID_POPUP_EDITFILE, OnPopupEditfile)
	ON_UPDATE_COMMAND_UI(ID_ALL_LEFT, OnUpdateAllLeft)
	ON_COMMAND(ID_ALL_LEFT, OnAllLeft)
	ON_UPDATE_COMMAND_UI(ID_ALL_RIGHT, OnUpdateAllRight)
	ON_COMMAND(ID_ALL_RIGHT, OnAllRight)
	ON_UPDATE_COMMAND_UI(ID_POPUP_COPYFROMOTHERSIDE, OnUpdateFromOther)
	ON_COMMAND(ID_POPUP_COPYFROMOTHERSIDE, OnFromOther)
	ON_UPDATE_COMMAND_UI(ID_POPUP_COPYALLDIFFSFROMOTHERSIDE, OnUpdateAllFromOther)
	ON_COMMAND(ID_POPUP_COPYALLDIFFSFROMOTHERSIDE, OnPopupCopyalldiffsfromotherside)
	ON_UPDATE_COMMAND_UI(ID_POPUP_COPYTOOTHERSIDE, OnUpdateToOther)
	ON_COMMAND(ID_POPUP_COPYTOOTHERSIDE, OnToOther)
	ON_UPDATE_COMMAND_UI(ID_POPUP_COPYALLDIFFSTOOTHERSIDE, OnUpdateAllToOther)
	ON_COMMAND(ID_POPUP_COPYALLDIFFSTOOTHERSIDE, OnPopupCopyalldiffstootherside)
	ON_WM_DRAWITEM()
	ON_WM_MOUSEWHEEL()
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CListViewEx::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CListViewEx::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CListViewEx::OnFilePrintPreview)
	ON_MESSAGE(WM_SETFONT, OnSetFont)
	ON_WM_MEASUREITEM_REFLECT( )
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDiffView construction/destruction

CDiffView::CDiffView()
{
	m_pList=NULL;
	m_nLines=0;
	m_bIsLeft=FALSE;
	m_nModifications=0;
}

CDiffView::~CDiffView()
{
}

BOOL CDiffView::PreCreateWindow(CREATESTRUCT& cs)
{
	cs.style |= LVS_SHOWSELALWAYS;

	return CListViewEx::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CDiffView drawing

void CDiffView::OnDraw(CDC* /*pDC*/)
{
	CMergeDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// TODO: add draw code for native data here
}

/////////////////////////////////////////////////////////////////////////////
// CDiffView printing

BOOL CDiffView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CDiffView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CDiffView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CDiffView diagnostics

#ifdef _DEBUG
void CDiffView::AssertValid() const
{
	CListViewEx::AssertValid();
}

void CDiffView::Dump(CDumpContext& dc) const
{
	CListViewEx::Dump(dc);
}

CMergeDoc* CDiffView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CMergeDoc)));
	return (CMergeDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CDiffView message handlers

void CDiffView::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* /*pScrollBar*/) 
{
	if (m_bIsLeft)
		mf->m_pRight->DoScroll(nSBCode, nPos, TRUE);
	else
		mf->m_pLeft->DoScroll(nSBCode, nPos, TRUE);
	DoScroll(nSBCode, nPos, FALSE);
}

void CDiffView::DoScroll(UINT code, UINT pos, BOOL bDoScroll)
{
	TRACE(_T("Scroll %s: pos=%d\n"), m_bIsLeft? _T("left"):_T("right"), pos);
	if (bDoScroll
		&& (code == SB_THUMBPOSITION
			|| code == SB_THUMBTRACK))
	{
		ScrollToLine(pos);
	}
	CListViewEx::OnVScroll(code, pos, NULL);
}

void CDiffView::OnLButtonDown(UINT /*nFlags*/, CPoint point) 
{
	SelectDiff(PointToDiff(point));
}

void CDiffView::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	CMergeDoc *pd = GetDocument();
	//if (nChar == VK_TAB)
	//{
	//	OnNextdiff();
	//}
	//else 
	if (nChar == VK_RIGHT)
	{
		OnL2R();
	}
	else if (nChar == VK_LEFT)
	{
		OnR2L();
	}
	else if (nChar == VK_HOME)
	{
		mf->m_pLeft->ScrollToLine(0);
		mf->m_pRight->ScrollToLine(0);
		SelectDiff(-1);
	}
	else if (nChar == VK_END)
	{
		mf->m_pLeft->ScrollToLine(m_nLines-1);
		mf->m_pRight->ScrollToLine(m_nLines-1);
		SelectDiff(-1);
	}
	else if (nChar == VK_NEXT)
	{
		int cnt = m_pList->GetCountPerPage();
		int top = m_pList->GetTopIndex();
		mf->m_pLeft->ScrollToLine(min((int)m_nLines-1, top+cnt-1));
		mf->m_pRight->ScrollToLine(min((int)m_nLines-1, top+cnt-1));
		SelectDiff(-1);
	}
	else if (nChar == VK_PRIOR)
	{
		int cnt = m_pList->GetCountPerPage();
		int top = m_pList->GetTopIndex();
		mf->m_pLeft->ScrollToLine(max(0, top-cnt+1));
		mf->m_pRight->ScrollToLine(max(0, top-cnt+1));
		SelectDiff(-1);
	}
	else 
		CListViewEx::OnKeyUp(nChar, nRepCnt, nFlags);
}

void CDiffView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	if (!(nChar == VK_TAB
		|| nChar == VK_RIGHT
		|| nChar == VK_LEFT))
		CListViewEx::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CDiffView::OnRButtonDown(UINT nFlags, CPoint point) 
{
	CListViewEx::OnRButtonDown(nFlags, point);
}

void CDiffView::OnContextMenu(CWnd* /*pWnd*/, CPoint point) 
{
	CPoint pt(point);
	ScreenToClient(&pt);
	if (PointToDiff(pt) == -1)
		SelectAll(FALSE);

	// CG: This block was added by the Pop-up Menu component
	{
		if (point.x == -1 && point.y == -1){
			//keystroke invocation
			CRect rect;
			GetClientRect(rect);
			ClientToScreen(rect);

			point = rect.TopLeft();
			point.Offset(5, 5);
		}

		CMenu menu;
		VERIFY(menu.LoadMenu(IDR_POPUP_DIFFVIEW));

		CMenu* pPopup = menu.GetSubMenu(0);
		ASSERT(pPopup != NULL);
		CWnd* pWndPopupOwner = this;

		while (pWndPopupOwner->GetStyle() & WS_CHILD)
			pWndPopupOwner = pWndPopupOwner->GetParent();

		pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y,
			pWndPopupOwner);
	}
}

void CDiffView::OnInitialUpdate() 
{
	CListViewEx::OnInitialUpdate();
	GetDocument()->m_pView = this;
	m_pList = &GetListCtrl();

	// set the font
	if (mf->m_bFontSpecified)
	{
		if (m_font.CreateFontIndirect(&mf->m_lfDiff))
			SetFont(&m_font);
	}


	m_pList->InsertColumn(0, _T(""), LVCFMT_LEFT, 800);
	SetExpandTabs(TRUE);
}

void CDiffView::DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct) 
{
	CMergeDoc *pd = GetDocument();
	CListCtrl& ListCtrl=GetListCtrl();
	CDC* pDC=CDC::FromHandle(lpDrawItemStruct->hDC);
	CRect rcItem(lpDrawItemStruct->rcItem);
	UINT uiFlags=ILD_TRANSPARENT;
	CImageList* pImageList;
	int nItem=lpDrawItemStruct->itemID;
	BOOL bFocus=(GetFocus()==this);
	COLORREF clrTextSave, clrBkSave;
	COLORREF clrImage=m_clrBkgnd;
	static TCHAR szBuff[MAX_PATH];
	LPCTSTR pszText;

// get item data

	LV_ITEM lvi;
	lvi.mask=LVIF_TEXT | LVIF_IMAGE | LVIF_STATE;
	lvi.iItem=nItem;
	lvi.iSubItem=0;
	lvi.pszText=szBuff;
	lvi.cchTextMax=sizeof(szBuff);
	lvi.stateMask=0xFFFF;		// get all state flags
	ListCtrl.GetItem(&lvi);

	BOOL bSelected=(bFocus || (GetStyle() & LVS_SHOWSELALWAYS)) && lvi.state & LVIS_SELECTED;
	bSelected=bSelected || (lvi.state & LVIS_DROPHILITED);

// set colors if item is selected

	CRect rcAllLabels;
	ListCtrl.GetItemRect(nItem,rcAllLabels,LVIR_BOUNDS);
	CRect rcLabel;
	ListCtrl.GetItemRect(nItem,rcLabel,LVIR_LABEL);
	rcAllLabels.left=rcLabel.left;
	if(m_bClientWidthSel && rcAllLabels.right<m_cxClient)
		rcAllLabels.right=m_cxClient;

	if(bSelected)
	{
		clrTextSave=pDC->SetTextColor(::GetSysColor(COLOR_HIGHLIGHTTEXT));
		clrBkSave=pDC->SetBkColor(::GetSysColor(COLOR_HIGHLIGHT));

		CBrush brush(::GetSysColor(COLOR_HIGHLIGHT));
		pDC->FillRect(rcAllLabels,&brush);
	}
	else
	{
		CRect rcClient, rcRow = rcItem;
		GetClientRect(&rcClient);
		rcRow.right = rcClient.right;
		
		COLORREF color=m_clrTextBk;
		int i = pd->LineToDiff(nItem);
		if (i >= 0)
		{
			switch(pd->m_diffs[i].op)
			{
			default:
				break;
			case OP_LEFTONLY:
				if (m_bIsLeft)
					color = RGB(255,255,92);
				else
					color = RGB(192,192,192);
				break;
			case OP_DIFF:
				if ((m_bIsLeft && (pd->m_diffs[i].blank0 <= 0 || nItem < pd->m_diffs[i].blank0))
					|| (!m_bIsLeft && (pd->m_diffs[i].blank1 <= 0 || nItem < pd->m_diffs[i].blank1)))
					color = RGB(255,255,92);
				else
					color = RGB(192,192,192);

				//color = RGB(255,255,92);
				break;
			case OP_RIGHTONLY:
				if (!m_bIsLeft)
					color = RGB(255,255,92);
				else
					color = RGB(192,192,192);
				break;
			}
		}
		CBrush brush(color);
		pDC->FillRect(rcRow, &brush);
	}

// set color and mask for the icon

	if(lvi.state & LVIS_CUT)
	{
		clrImage=m_clrBkgnd;
		uiFlags|=ILD_BLEND50;
	}
	else if(bSelected)
	{
		clrImage=::GetSysColor(COLOR_HIGHLIGHT);
		uiFlags|=ILD_BLEND50;
	}

// draw state icon

	UINT nStateImageMask=lvi.state & LVIS_STATEIMAGEMASK;
	if(nStateImageMask)
	{
		int nImage=(nStateImageMask>>12)-1;
		pImageList=ListCtrl.GetImageList(LVSIL_STATE);
		if(pImageList)
			pImageList->Draw(pDC,nImage,CPoint(rcItem.left,rcItem.top),ILD_TRANSPARENT);
	}

// draw normal and overlay icon

	CRect rcIcon;
	ListCtrl.GetItemRect(nItem,rcIcon,LVIR_ICON);

	pImageList=ListCtrl.GetImageList(LVSIL_SMALL);
	if(pImageList)
	{
		UINT nOvlImageMask=lvi.state & LVIS_OVERLAYMASK;
		if(rcItem.left<rcItem.right-1)
			ImageList_DrawEx(pImageList->m_hImageList,lvi.iImage,pDC->m_hDC,rcIcon.left,rcIcon.top,16,16,m_clrBkgnd,clrImage,uiFlags | nOvlImageMask);
	}

	// draw item label
	ListCtrl.GetItemRect(nItem,rcItem,LVIR_LABEL);
	rcItem.right-=m_cxStateImageOffset;

	pszText=MakeShortString(pDC,szBuff,rcItem.right-rcItem.left,2*OFFSET_FIRST);

	rcLabel=rcItem;
	rcLabel.left+=OFFSET_FIRST;
	rcLabel.right-=OFFSET_FIRST;

	pDC->DrawText(pszText,-1,rcLabel, m_nTabFlag | DT_LEFT | DT_SINGLELINE | DT_NOPREFIX | DT_NOCLIP | DT_VCENTER);

	// draw labels for extra columns
	LV_COLUMN lvc;
	lvc.mask=LVCF_FMT | LVCF_WIDTH;

	for(int nColumn=1; ListCtrl.GetColumn(nColumn,&lvc); nColumn++)
	{
		rcItem.left=rcItem.right;
		rcItem.right+=lvc.cx;

		int nRetLen=ListCtrl.GetItemText(nItem,nColumn,szBuff,sizeof(szBuff));
		if(nRetLen==0) continue;

		pszText=MakeShortString(pDC,szBuff,rcItem.right-rcItem.left,2*OFFSET_OTHER);

		UINT nJustify=DT_LEFT;

		if(pszText==szBuff)
		{
			switch(lvc.fmt & LVCFMT_JUSTIFYMASK)
			{
			case LVCFMT_RIGHT:
				nJustify=DT_RIGHT;
				break;
			case LVCFMT_CENTER:
				nJustify=DT_CENTER;
				break;
			default:
				break;
			}
		}

		rcLabel=rcItem;
		rcLabel.left+=OFFSET_OTHER;
		rcLabel.right-=OFFSET_OTHER;

		pDC->DrawText(pszText,-1,rcLabel,nJustify | DT_SINGLELINE | DT_NOPREFIX | DT_NOCLIP | DT_VCENTER);
	}

// draw focus rectangle if item has focus

	if(lvi.state & LVIS_FOCUSED && bFocus)
		pDC->DrawFocusRect(rcAllLabels);

// set original colors if item was selected

	if(bSelected)
	{
	        pDC->SetTextColor(clrTextSave);
		pDC->SetBkColor(clrBkSave);
	}
}

BOOL CDiffView::PrimeListWithFile(LPCTSTR szFilename)
{
	CMergeDoc *pd = GetDocument();
	CTextFile file;
	if (file.Open(szFilename, CFile::modeRead))
	{
		register int i;
		TCHAR buf[2048];
		LPTSTR p;
		int nDiff=0, nReadLine=0, lines0=0, lines1=0;
		int begin0,end0,begin1,end1;
		bool shiftdiff=FALSE;

		m_strFile = szFilename;

		m_pList->SendMessage(WM_SETREDRAW, FALSE, 0L);
		m_pList->DeleteAllItems();
		m_nLines=0;
		if (nDiff < (int)pd->m_nDiffs)
		{
			begin0=pd->m_diffs[nDiff].begin0;
			end0=pd->m_diffs[nDiff].end0;
			begin1=pd->m_diffs[nDiff].begin1;
			end1=pd->m_diffs[nDiff].end1;
			lines0 = end0-begin0+1;
			lines1 = end1-begin1+1;
		}
		while (file.ReadLine(buf, sizeof buf))
		{
			if (nDiff >= (int)pd->m_nDiffs
				|| ((m_bIsLeft && nReadLine < begin0)||(!m_bIsLeft && nReadLine < begin1)))
			{
				p = _tcstok(buf,_T("\r\n"));
				AddItem(m_nLines, 0, p);
				//AddItem(m_nLines, 0, pd->ExpandTabs(p));
				m_pList->SetItemData(m_nLines, 0);
				m_nLines++;
			}
			else 
			{
				if (m_bIsLeft && begin0 == nReadLine)
					pd->m_diffs[nDiff].dbegin0 = pd->m_diffs[nDiff].dend0 = m_nLines;
				else if (!m_bIsLeft && begin1 == nReadLine)
					pd->m_diffs[nDiff].dbegin1 = pd->m_diffs[nDiff].dend1 = m_nLines;

				if (pd->m_diffs[nDiff].op == OP_RIGHTONLY)
				{
					if (m_bIsLeft)
					{
						// write blanks for missing lines
						if (begin0 == nReadLine)
						{
							pd->m_diffs[nDiff].blank0 = m_nLines;
							for (i=0; i < lines1; i++, m_nLines++)
							{
								AddItem(m_nLines, 0, _T(""));
								m_pList->SetItemData(m_nLines, 1);
							}
							shiftdiff=TRUE;
						}
						p = _tcstok(buf,_T("\r\n"));
						AddItem(m_nLines, 0, p);
						//AddItem(m_nLines, 0, pd->ExpandTabs(p));
						m_pList->SetItemData(m_nLines, 0);
						m_nLines++;
					}
					else 
					{
						p = _tcstok(buf,_T("\r\n"));
						AddItem(m_nLines, 0, p);
						//AddItem(m_nLines, 0, pd->ExpandTabs(p));
						m_pList->SetItemData(m_nLines, 1);
						m_nLines++;
						if (nReadLine == end1)
						{
							pd->m_diffs[nDiff].blank1 = -1;
							shiftdiff=TRUE;
						}
					}
					
					if (m_bIsLeft && nReadLine == begin0)
						pd->m_diffs[nDiff].dend0 = m_nLines-2;
					else if (!m_bIsLeft && nReadLine == end1)
						pd->m_diffs[nDiff].dend1 = m_nLines-1;
				}
				else if (pd->m_diffs[nDiff].op == OP_LEFTONLY)
				{
					if (!m_bIsLeft)
					{
						// write blanks for missing lines
						if (begin1 == nReadLine)
						{
							pd->m_diffs[nDiff].blank1 = m_nLines;
							for (i=0; i < lines0; i++, m_nLines++)
							{
								AddItem(m_nLines, 0, _T(""));
								m_pList->SetItemData(m_nLines, 1);
							}
							shiftdiff=TRUE;
						}
						p = _tcstok(buf,_T("\r\n"));
						AddItem(m_nLines, 0, p);
						//AddItem(m_nLines, 0, pd->ExpandTabs(p));
						m_pList->SetItemData(m_nLines, 0);
						m_nLines++;
					}
					else 
					{
						p = _tcstok(buf,_T("\r\n"));
						AddItem(m_nLines, 0, p);
						//AddItem(m_nLines, 0, pd->ExpandTabs(p));
						m_pList->SetItemData(m_nLines, 1);
						m_nLines++;
						if (nReadLine == end0)
						{
							pd->m_diffs[nDiff].blank0 = -1;
							shiftdiff=TRUE;
						}
					}
					
					if (m_bIsLeft && nReadLine == end0)
						pd->m_diffs[nDiff].dend0 = m_nLines-1;
					else if (!m_bIsLeft && nReadLine == begin1)
						pd->m_diffs[nDiff].dend1 = m_nLines-2;
				}
				else // OP_DIFF
				{
					if (m_bIsLeft)
					{
						p = _tcstok(buf,_T("\r\n"));
						AddItem(m_nLines, 0, p);
						//AddItem(m_nLines, 0, pd->ExpandTabs(p));
						m_pList->SetItemData(m_nLines, 1);
						m_nLines++;
						if (nReadLine == end0)
						{
							if (lines1>lines0)
							{
								pd->m_diffs[nDiff].blank0 = m_nLines;
								for (i=0; i < lines1-lines0; i++, m_nLines++)
								{
									AddItem(m_nLines, 0, _T(""));
									m_pList->SetItemData(m_nLines, 1);
								}
							}
							shiftdiff=TRUE;
						}
					}
					else 
					{
						p = _tcstok(buf,_T("\r\n"));
						AddItem(m_nLines, 0, p);
						//AddItem(m_nLines, 0, pd->ExpandTabs(p));
						m_pList->SetItemData(m_nLines, 1);
						m_nLines++;
						if (nReadLine == end1)
						{
							if (lines0>lines1)
							{
								pd->m_diffs[nDiff].blank1 = m_nLines;
								for (i=0; i < lines0-lines1; i++, m_nLines++)
								{
									AddItem(m_nLines, 0, _T(""));
									m_pList->SetItemData(m_nLines, 1);
								}
							}
							shiftdiff=TRUE;
						}
					}
					
					if (m_bIsLeft && end0 == nReadLine)
						pd->m_diffs[nDiff].dend0 = m_nLines-1;
					else if (!m_bIsLeft && end1 == nReadLine)
						pd->m_diffs[nDiff].dend1 = m_nLines-1;
				}
			}
			
			if (shiftdiff)
			{
				nDiff++;
				shiftdiff=FALSE;
				if (nDiff < (int)pd->m_nDiffs)
				{
					begin0=pd->m_diffs[nDiff].begin0;
					end0=pd->m_diffs[nDiff].end0;
					begin1=pd->m_diffs[nDiff].begin1;
					end1=pd->m_diffs[nDiff].end1;
					lines0 = end0-begin0+1;
					lines1 = end1-begin1+1;
				}
			}
			nReadLine++;
		}

		file.Close();
		m_pList->SendMessage(WM_SETREDRAW, TRUE, 0L);

		TRACE(_T("%s\n"), m_bIsLeft? _T("LEFT"):_T("RIGHT"));
		for (i=0; i < (int)pd->m_nDiffs; i++)
		{
			if (m_bIsLeft)
				TRACE(_T("%d: %d,%d\n"), i, pd->m_diffs[i].dbegin0, pd->m_diffs[i].dend0);
			else
				TRACE(_T("%d: %d,%d\n"), i, pd->m_diffs[i].dbegin1, pd->m_diffs[i].dend1);
		}

		SetColumnText(szFilename);
		ResetMod();

		if (mf->m_bScrollToFirst && pd->m_nDiffs > 0)
			ScrollToLine(pd->m_diffs[0].dbegin0);
	}
	else
		AfxMessageBox(_T("Failed to open file"));
	return m_nLines>0;
}

void CDiffView::SetColumnText(LPCTSTR szText)
{
	LV_COLUMN col;

	col.mask = LVCF_TEXT;
	TCHAR temp[MAX_PATH];
	_tcscpy(temp, szText);
	col.pszText = temp;

	GetListCtrl().SetColumn(0,&col);
}

int CDiffView::PointToDiff(CPoint & pt)
{
	CMergeDoc* pd = GetDocument();
	int sel = m_pList->HitTest(pt);
	if (sel != -1)
	{
		for (UINT i=0; i < pd->m_nDiffs; i++)
			if (sel >= (int)pd->m_diffs[i].dbegin0
				&& sel <= (int)pd->m_diffs[i].dend0)
				return i;
	}
	return -1;
}

void CDiffView::SelectAll(BOOL bSelected)
{
	if (bSelected)
	{
		for (int i=0; i < m_pList->GetItemCount(); i++)
			m_pList->SetItemState(i, LVIS_SELECTED, LVIS_SELECTED);
	}
	else
	{
		int sel=-1;
		while ((sel=m_pList->GetNextItem(sel, LVNI_SELECTED)) != -1)
			m_pList->SetItemState(sel, 0, LVIS_SELECTED);
	}
}


void CDiffView::NextDiff()
{
	CMergeDoc *pd = GetDocument();
	int cnt = m_pList->GetItemCount();
	if (cnt <= 0)
		return;

	CRect rc;
	m_pList->GetItemRect(0, &rc, LVIR_BOUNDS);

	int sel,curpos, pos;
	if ((sel=m_pList->GetNextItem(-1, LVNI_SELECTED)) != -1)
	{
		pos = sel+1;
		while ((sel=m_pList->GetNextItem(sel, LVNI_SELECTED)) != -1)
			pos = sel+1;
		
		// now, move to the next diff
		curpos = pos;
		while (pos < cnt-1
			&& (sel=m_pList->GetItemData(pos)) == 0)
			pos++;

		if (curpos == pos)
			return;
	}
	else
	{
		curpos = pos = m_pList->GetTopIndex();

		int nDiff = pd->LineToDiff(pos)+1;
		if (nDiff > 0)
		{
			if (nDiff < (int)pd->m_nDiffs)
				pos = pd->m_diffs[nDiff].dbegin0;
			else
				pos = cnt-1;
		}
		else
		{
			// now, move to the next diff
			while (pos < cnt-1
				&& m_pList->GetItemData(pos) == 0)
				pos++;
		}
	}

	// scroll to pos
	//int pix = (pos - curpos) * rc.Height();
	//mf->m_pLeft->m_pList->Scroll(CSize(0,pix)); 
	//mf->m_pRight->m_pList->Scroll(CSize(0,pix)); 
	mf->m_pLeft->ScrollToLine(pos);
	mf->m_pRight->ScrollToLine(pos);

	SelectDiff(pd->LineToDiff(pos));
}


void CDiffView::OnPopupSave() 
{
   // PV: The following lines were added if we have a "/Diff" command line
   if(g_bMergeRightOnly && m_bIsLeft)
      return;
   if(g_bMergeLeftOnly && !m_bIsLeft)
      return;
   if(!g_bDoDifferences)
   	GetDocument()->DoSave(m_strFile, m_pList, m_bIsLeft);
}

void CDiffView::OnUpdatePopupSave(CCmdUI* pCmdUI) 
{
   // PV: The following lines were added if we have a "/Diff" command line
   if(g_bMergeRightOnly && m_bIsLeft)
      pCmdUI->Enable(false);
   else if(g_bMergeLeftOnly && !m_bIsLeft)
      pCmdUI->Enable(false);
   else if(g_bDoDifferences)
      pCmdUI->Enable(false);
   else
	   pCmdUI->Enable(IsModified());
}

void CDiffView::SelectDiff(int nDiff)
{
	CMergeDoc *pd = GetDocument();
	mf->m_pLeft->SelectAll(FALSE);
	mf->m_pRight->SelectAll(FALSE);
	if (nDiff >= 0
		&& nDiff < (int)pd->m_nDiffs)
	{
		for (UINT i=pd->m_diffs[nDiff].dbegin0; i <= pd->m_diffs[nDiff].dend0; i++)
		{
			mf->m_pLeft->m_pList->SetItemState(i, LVIS_SELECTED, LVIS_SELECTED);
			mf->m_pRight->m_pList->SetItemState(i, LVIS_SELECTED, LVIS_SELECTED);
		}
	}
}

void CDiffView::OnL2R() 
{
   if(!g_bMergeLeftOnly  && !g_bDoDifferences)
	   GetDocument()->ListCopy(mf->m_pLeft, mf->m_pRight);
}

void CDiffView::OnUpdateL2R(CCmdUI* pCmdUI) 
{
   // PV: The following lines were added if we have a "/Diff" command line
   if(g_bMergeLeftOnly)
      pCmdUI->Enable(false);
   else if(g_bDoDifferences)
      pCmdUI->Enable(false);
   else
   	pCmdUI->Enable(mf->m_pLeft && mf->m_pLeft->m_pList->GetSelectedCount()>0);
	//TRACE(_T("OnUpdateL2R\n"));
}

void CDiffView::OnR2L() 
{
   if(!g_bMergeRightOnly  && !g_bDoDifferences)
   	GetDocument()->ListCopy(mf->m_pRight, mf->m_pLeft);
}

void CDiffView::OnUpdateR2L(CCmdUI* pCmdUI) 
{
   // PV: The following lines were added if we have a "/Diff" command line
   if(g_bMergeRightOnly)
      pCmdUI->Enable(false);
   else if(g_bDoDifferences)
      pCmdUI->Enable(false);
   else
   	pCmdUI->Enable(mf->m_pRight && mf->m_pRight->m_pList->GetSelectedCount()>0);
	//TRACE(_T("OnUpdateR2L\n"));
}

void CDiffView::OnUndo() 
{
	GetDocument()->Undo();
}

void CDiffView::OnUpdateUndo(CCmdUI* pCmdUI) 
{
   // PV: The following lines were added if we have a "/Diff" command line
   if(g_bMergeRightOnly && m_bIsLeft)
      pCmdUI->Enable(false);
   else if(g_bMergeLeftOnly && !m_bIsLeft)
      pCmdUI->Enable(false);
   else if(g_bDoDifferences)
      pCmdUI->Enable(false);
   else
	pCmdUI->Enable(!GetDocument()->m_undoList.IsEmpty());
}

void CDiffView::ScrollToLine(UINT nLine)
{
	CRect rc;
	m_pList->GetItemRect(0, &rc, LVIR_BOUNDS);

	int pix = (nLine - m_pList->GetTopIndex()) * rc.Height();
	if (pix != 0)
		m_pList->Scroll(CSize(0,pix)); 
}

void CDiffView::ResetMod(int nMods /*=0*/)
{
	m_nModifications = nMods;
}


void CDiffView::AddMod()
{
	m_nModifications++;
}

void CDiffView::SubMod()
{
	m_nModifications = max(0, m_nModifications-1);
}

BOOL CDiffView::IsModified()
{
   if(!g_bDoDifferences)
      return (m_nModifications>0);
   else
      return false;
}

void CDiffView::OnFileSave() 
{
   // PV: The following lines were added if we have a "/Diff" command line
   if(!g_bDoDifferences)
   	SaveHelper();
}

void CDiffView::OnUpdateFileSave(CCmdUI* pCmdUI) 
{
   // PV: The following lines were added if we have a "/Diff" command line
   if(g_bDoDifferences)
   	pCmdUI->Enable(false);
   else
	   pCmdUI->Enable((mf->m_pLeft && mf->m_pLeft->IsModified()) || (mf->m_pRight && mf->m_pRight->IsModified()));
}

void CDiffView::OnNextdiff() 
{
	NextDiff();	
}

void CDiffView::OnUpdateNextdiff(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(TRUE);
}

void CDiffView::PrevDiff()
{
	CMergeDoc *pd = GetDocument();
	int cnt = m_pList->GetItemCount();
	if (cnt <= 0)
		return;

	CRect rc;
	m_pList->GetItemRect(0, &rc, LVIR_BOUNDS);

	int sel,curpos, pos;
	if ((sel=m_pList->GetNextItem(-1, LVNI_SELECTED)) != -1)
	{
		pos = sel-1;
		
		// now, move to the prev diff
		curpos = pos;
		while (pos > 0
			&& m_pList->GetItemData(pos) == 0)
			pos--;

		if (curpos == pos)
			return;
	}
	else
	{
		curpos = pos = m_pList->GetTopIndex();

		int nDiff = pd->LineToDiff(pos)-1;
		if (nDiff > 0)
		{
			if (nDiff >= 0)
				pos = pd->m_diffs[nDiff].dbegin0;
			else
				pos = 0;
		}
		else
		{
			// now, move to the next diff
			while (pos > 0
				&& m_pList->GetItemData(pos) == 0)
				pos--;
		}
	}

	// scroll to pos
	//int pix = (pos - curpos) * rc.Height();
	//mf->m_pLeft->m_pList->Scroll(CSize(0,pix)); 
	//mf->m_pRight->m_pList->Scroll(CSize(0,pix)); 
	mf->m_pLeft->ScrollToLine(pos);
	mf->m_pRight->ScrollToLine(pos);

	SelectDiff(pd->LineToDiff(pos));
}

void CDiffView::OnPrevdiff() 
{
	PrevDiff();
}

void CDiffView::OnUpdatePrevdiff(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(TRUE);
}


/*void CDiffView::OnEditleft() 
{
	CEditFile dlg(mf->m_pLeft->m_strFile, GetDocument());
	dlg.m_nStartLine = mf->m_pLeft->m_pList->GetTopIndex();
	dlg.DoModal();
}

void CDiffView::OnEditright() 
{
	CEditFile dlg(mf->m_pRight->m_strFile, GetDocument());
	dlg.m_nStartLine = mf->m_pRight->m_pList->GetTopIndex();
	dlg.DoModal();
}*/

void CDiffView::OnPopupEditfile() 
{
	BeginWaitCursor();
	CMergeDoc *pd = GetDocument();
	CEditFile dlg(m_strFile,pd);
	dlg.m_nStartLine = m_pList->GetTopIndex();
	
	dlg.m_nStartLine -= pd->CountPrevBlanks(dlg.m_nStartLine, m_bIsLeft);

	CString s,s2;
	int idx,cnt = m_pList->GetItemCount();
	for (int i=0; i < cnt; i++)
	{
		idx=pd->LineToDiff(i);
		s = m_pList->GetItemText(i,0);
		if (idx==-1
			//|| m_diffs[idx].op == OP_DIFF
			|| (m_bIsLeft && (pd->m_diffs[idx].blank0 <= 0 || i < pd->m_diffs[idx].blank0))
			|| (!m_bIsLeft && (pd->m_diffs[idx].blank1 <= 0 || i < pd->m_diffs[idx].blank1)))
		{
			s2.Format(_T("%s\r\n"), s);
			dlg.m_strText += s2;
		}
	}
	EndWaitCursor();
	if (dlg.DoModal()==IDOK)
	{
		pd->Rescan();
		mf->m_pLeft->ScrollToLine(dlg.m_nStartLine);
		mf->m_pRight->ScrollToLine(dlg.m_nStartLine);
		ResetMod();
	}
}

BOOL CDiffView::SaveHelper()
{
	BOOL result = TRUE;
	CMergeDoc *pd = GetDocument();
	CString s;
   if(g_bDoDifferences)
      return TRUE;

	AfxFormatString1(s, IDS_SAVE_FMT, pd->m_strLeftFile); 
	if (mf->m_pLeft
		&& mf->m_pLeft->IsModified()  &&  !g_bMergeRightOnly)
	{
		switch(AfxMessageBox(s, MB_YESNOCANCEL|MB_ICONQUESTION))
		{
		case IDYES:
			if (pd->DoSave(pd->m_strLeftFile, mf->m_pLeft->m_pList, TRUE))
				mf->m_pLeft->ResetMod();
			else
				result=FALSE;
			break;
		case IDNO:
			break;
		default:  // IDCANCEL
			result=FALSE;
			break;
		}
	}

	AfxFormatString1(s, IDS_SAVE_FMT, pd->m_strRightFile); 
	if (mf->m_pRight
		&& mf->m_pRight->IsModified()  &&  !g_bMergeLeftOnly)
	{
		switch(AfxMessageBox(s, MB_YESNOCANCEL|MB_ICONQUESTION))
		{
		case IDYES:
			if (pd->DoSave(pd->m_strRightFile, mf->m_pRight->m_pList, FALSE))
				mf->m_pRight->ResetMod();
			else
				result=FALSE;
			break;
		case IDNO:
			break;
		default:  // IDCANCEL
			result=FALSE;
			break;
		}
	}
	return result;
}

void CDiffView::OnUpdateEditFile(CCmdUI* pCmdUI)
{
   // PV: The following lines were added if we have a "/Diff" command line
   if(g_bMergeRightOnly && m_bIsLeft)
      pCmdUI->Enable(false);
   else if(g_bMergeLeftOnly && !m_bIsLeft)
      pCmdUI->Enable(false);
   else if(g_bDoDifferences)
      pCmdUI->Enable(false);
   else
      pCmdUI->Enable(true);
}

void CDiffView::OnUpdateAllLeft(CCmdUI* pCmdUI)
{
   // PV: The following lines were added if we have a "/Diff" command line
   if(g_bMergeRightOnly)
      pCmdUI->Enable(false);
   else if(g_bDoDifferences)
      pCmdUI->Enable(false);
   else
      pCmdUI->Enable(true);
}

void CDiffView::OnAllLeft() 
{
	if (AfxMessageBox(IDS_CONFIRM_ALL_LEFT, MB_YESNO|MB_ICONQUESTION)==IDYES)
	{		
		CMergeDoc *pd = GetDocument();

		if (mf->SyncFiles(pd->m_strRightFile, pd->m_strLeftFile))
		{
			GetParentFrame()->PostMessage(WM_CLOSE);
			mf->UpdateCurrentFileStatus(FILE_SAME);
		}
	}
}

void CDiffView::OnUpdateAllRight(CCmdUI* pCmdUI)
{
   // PV: The following lines were added if we have a "/Diff" command line
   if(g_bMergeLeftOnly)
      pCmdUI->Enable(false);
   else if(g_bDoDifferences)
      pCmdUI->Enable(false);
   else
      pCmdUI->Enable(true);
}

void CDiffView::OnAllRight() 
{
	if (AfxMessageBox(IDS_CONFIRM_ALL_RIGHT, MB_YESNO|MB_ICONQUESTION)==IDYES)
	{		
		CMergeDoc *pd = GetDocument();

		if (mf->SyncFiles(pd->m_strLeftFile, pd->m_strRightFile))
		{
			GetParentFrame()->PostMessage(WM_CLOSE);
			mf->UpdateCurrentFileStatus(FILE_SAME);
		}
	}
}

LRESULT CDiffView::OnSetFont(WPARAM /*wParam*/, LPARAM)
{
        LRESULT res =  Default();

        CRect rc;
        GetWindowRect( &rc );

        WINDOWPOS wp;
        wp.hwnd = m_hWnd;
        wp.cx = rc.Width();
        wp.cy = rc.Height();
        wp.flags = SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOOWNERZORDER | SWP_NOZORDER;
        SendMessage( WM_WINDOWPOSCHANGED, 0, (LPARAM)&wp );

        return res;
}

void CDiffView::MeasureItem ( LPMEASUREITEMSTRUCT lpMeasureItemStruct )
{
	CClientDC dc( this );
	CFont* pFont = GetFont();
	ASSERT( pFont );
	
	CFont *oldfont = dc.SelectObject( pFont );
	
	TEXTMETRIC txtMetric;
	BOOL bRet = dc.GetTextMetrics( &txtMetric );
	ASSERT( bRet );
	
	int cyChar = txtMetric.tmHeight + txtMetric.tmExternalLeading;
	ASSERT( 0 != cyChar );
	
	lpMeasureItemStruct->itemHeight = cyChar;
	if (oldfont != NULL)
		dc.SelectObject(oldfont);
}

void CDiffView::OnUpdateFromOther(CCmdUI* pCmdUI)
{
   if(m_bIsLeft)
      OnUpdateR2L(pCmdUI);
   else
      OnUpdateL2R(pCmdUI);
}

void CDiffView::OnFromOther()
{
   if(m_bIsLeft)
      OnR2L();
   else
      OnL2R();
}

void CDiffView::OnUpdateAllFromOther(CCmdUI* pCmdUI)
{
   if(g_bMergeRightOnly && m_bIsLeft)
      pCmdUI->Enable(false);
   else if(g_bMergeLeftOnly && !m_bIsLeft)
      pCmdUI->Enable(false);
   else if(g_bDoDifferences)
      pCmdUI->Enable(false);
   else
      pCmdUI->Enable(true);
}

void CDiffView::OnPopupCopyalldiffsfromotherside() 
{
	if (m_bIsLeft)
		OnAllLeft();
	else
		OnAllRight();
}

void CDiffView::OnUpdateToOther(CCmdUI* pCmdUI)
{
   if(m_bIsLeft)
      OnUpdateL2R(pCmdUI);
   else
      OnUpdateR2L(pCmdUI);
}

void CDiffView::OnToOther()
{
   if(m_bIsLeft)
      OnL2R();
   else
      OnR2L();
}

void CDiffView::OnUpdateAllToOther(CCmdUI* pCmdUI)
{
   if(g_bMergeRightOnly && !m_bIsLeft)
      pCmdUI->Enable(false);
   else if(g_bMergeLeftOnly && m_bIsLeft)
      pCmdUI->Enable(false);
   else if(g_bDoDifferences)
      pCmdUI->Enable(false);
   else
      pCmdUI->Enable(true);
}

void CDiffView::OnPopupCopyalldiffstootherside() 
{
	if (m_bIsLeft)
		OnAllRight();
	else
		OnAllLeft();
}

void CDiffView::UpdateResources()
{

}

BOOL CDiffView::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt) 
{
	if (m_bIsLeft)
		mf->m_pRight->DoMouseWheel(nFlags, zDelta, pt);
	else
		mf->m_pLeft->DoMouseWheel(nFlags, zDelta, pt);
	return DoMouseWheel(nFlags, zDelta, pt);
}

BOOL CDiffView::DoMouseWheel(UINT nFlags, short zDelta, CPoint pt)
{
	return CListViewEx::OnMouseWheel(nFlags, zDelta, pt);
}
