/////////////////////////////////////////////////////////////////////////////
//    WinMerge:  an interactive diff/merge utility
//    Copyright (C) 1997  Dean P. Grimm
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
/////////////////////////////////////////////////////////////////////////////
// DiffView.h : interface of the CDiffView class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_DIFFVIEW_H__BBCD4F92_34E4_11D1_BAA6_00A024706EDC__INCLUDED_)
#define AFX_DIFFVIEW_H__BBCD4F92_34E4_11D1_BAA6_00A024706EDC__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000
#include "listvwex.h"
#include "afxtempl.h"
#include "MergeDoc.h"


class CDiffView : public CListViewEx
{
protected: // create from serialization only
	CString m_strFile;
	CDiffView();
	CFont m_font;
	DECLARE_DYNCREATE(CDiffView)

// Attributes
public:
	CMergeDoc* GetDocument();

// Operations
public:
	void SelectDiff(int nDiff);
	void NextDiff();
	void SelectAll(BOOL bSelected = TRUE);
	int PointToDiff(CPoint& pt);
	void SetColumnText(LPCTSTR szText);
	void DoScroll(UINT code, UINT pos, BOOL bDoScroll);
	CListCtrl * m_pList;
	BOOL m_bIsLeft;
	UINT m_nLines;
	BOOL PrimeListWithFile(LPCTSTR szFilename);

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CDiffView)
	public:
	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	virtual void OnInitialUpdate();
	protected:
	virtual BOOL OnPreparePrinting(CPrintInfo* pInfo);
	virtual void OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnEndPrinting(CDC* pDC, CPrintInfo* pInfo);
	//}}AFX_VIRTUAL

// Implementation
public:
	BOOL DoMouseWheel(UINT nFlags, short zDelta, CPoint pt);
	void UpdateResources();
	BOOL SaveHelper();
	void PrevDiff();
	void SubMod();
	void AddMod();
	BOOL IsModified();
	void ResetMod(int nMods =0);
	void ScrollToLine(UINT nLine);
	virtual ~CDiffView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:
	virtual void DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct);

// Generated message map functions
protected:
	int m_nModifications;
	//{{AFX_MSG(CDiffView)	
	afx_msg void OnFileSave();
	afx_msg void OnUpdateFileSave(CCmdUI* pCmdUI);
	afx_msg void OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnRButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnContextMenu(CWnd* pWnd, CPoint point);
	afx_msg void OnPopupSave();
	afx_msg void OnL2R();
	afx_msg void OnUpdateL2R(CCmdUI* pCmdUI);
	afx_msg void OnR2L();
	afx_msg void OnUpdateR2L(CCmdUI* pCmdUI);
	afx_msg void OnUndo();
	afx_msg void OnUpdateUndo(CCmdUI* pCmdUI);
	afx_msg void OnUpdatePopupSave(CCmdUI* pCmdUI);
	afx_msg void OnNextdiff();
	afx_msg void OnUpdateNextdiff(CCmdUI* pCmdUI);
	afx_msg void OnPrevdiff();
	afx_msg void OnUpdatePrevdiff(CCmdUI* pCmdUI);
	afx_msg void OnPopupEditfile();
	afx_msg void OnUpdateEditFile(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAllLeft(CCmdUI* pCmdUI);
	afx_msg void OnAllLeft();
	afx_msg void OnUpdateAllRight(CCmdUI* pCmdUI);
	afx_msg void OnAllRight();
	afx_msg void OnUpdateFromOther(CCmdUI* pCmdUI);
	afx_msg void OnFromOther();
	afx_msg void OnUpdateAllFromOther(CCmdUI* pCmdUI);
	afx_msg void OnPopupCopyalldiffsfromotherside();
	afx_msg void OnUpdateToOther(CCmdUI* pCmdUI);
	afx_msg void OnToOther();
	afx_msg void OnUpdateAllToOther(CCmdUI* pCmdUI);
	afx_msg void OnPopupCopyalldiffstootherside();
	afx_msg BOOL OnMouseWheel(UINT nFlags, short zDelta, CPoint pt);
	//}}AFX_MSG
	afx_msg LRESULT OnSetFont(WPARAM wParam, LPARAM);
	afx_msg void MeasureItem ( LPMEASUREITEMSTRUCT lpMeasureItemStruct );
	DECLARE_MESSAGE_MAP()
};

#ifndef _DEBUG  // debug version in DiffView.cpp
inline CMergeDoc* CDiffView::GetDocument()
   { return (CMergeDoc*)m_pDocument; }
#endif

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_DIFFVIEW_H__BBCD4F92_34E4_11D1_BAA6_00A024706EDC__INCLUDED_)
