//////////////////////////////////////////////////////////////////////
// StringEx.h
//
// Adapted and modified from code posted by "Zafir"?
// http://codeguru.developer.com/string/cstring_ext.shtml

#ifndef __STRINGEX_H_
#define __STRINGEX_H_

class CStringEx : public CString
{
public:	CStringEx() : CString( ){};

	CStringEx( const CString& stringSrc) : CString( stringSrc ){};
	CStringEx( const CStringEx& stringSrc) : CString( stringSrc ){};
	CStringEx( TCHAR ch, int nRepeat = 1 ) : CString( ch, nRepeat ){};
	CStringEx( LPCTSTR lpch, int nLength ) : CString( lpch, nLength ){};
	CStringEx( const unsigned char* psz ) : CString( psz ){};
	CStringEx( LPCWSTR lpsz ) : CString( lpsz ){};
	CStringEx( LPCSTR lpsz ) : CString( lpsz ){};

	CStringEx& Insert(int pos, LPCTSTR s);
	CStringEx& Insert(int pos, TCHAR c);

	CStringEx& Delete(int pos, int len);
	CStringEx& Replace(int pos, int len, LPCTSTR s);

	int Find( TCHAR ch, int startpos = 0 ) const;
	int Find( LPCTSTR lpszSub, int startpos = 0 ) const;
	int FindNoCase( TCHAR ch, int startpos = 0 ) const;
	int FindNoCase( LPCTSTR lpszSub, int startpos = 0 ) const;

	int FindReplace( LPCTSTR lpszSub, LPCTSTR lpszReplaceWith, BOOL bGlobal = TRUE );
	int FindReplaceNoCase( LPCTSTR lpszSub, LPCTSTR lpszReplaceWith, 
				BOOL bGlobal = TRUE );

  bool IsNumeric(TCHAR ch);
  bool IsNumeric();

	CString RemoveFirst(LPCTSTR substr); //Larry
	CString RemoveLast(LPCTSTR substr); //Larry
	CString RemoveFirstNoCase(LPCTSTR substr); //Larry
	CString RemoveLastNoCase(LPCTSTR substr); //Larry

	int ReverseFind( TCHAR ch ) const{ return CString::ReverseFind(ch);};
	int ReverseFind( LPCTSTR lpszSub, int startpos = -1 ) const;
	int ReverseFindNoCase( TCHAR ch, int startpos = -1  ) const;
	int ReverseFindNoCase( LPCTSTR lpszSub, int startpos = -1 ) const;

	CStringEx GetField( LPCTSTR delim, int fieldnum);
  CStringEx GetFieldReverse( LPCTSTR delim, int fieldnum); //Larry

	CStringEx GetField( TCHAR delim, int fieldnum);
  CStringEx GetFieldReverse( TCHAR delim, int fieldnum); //Larry

	int GetFieldCount( LPCTSTR delim );
	int GetFieldCount( TCHAR delim );

	CStringEx GetDelimitedField( LPCTSTR delimStart, LPCTSTR delimEnd, 
				int fieldnum = 0);
};
				
#endif
/////////////////////////////////////////////////////////////////////

// Adapted and modified from code posted by "Zafir"?
// http://codeguru.developer.com/string/cstring_ext.shtml

// NOTES and EXAMPLES
/////////////////////////////////////////////////////////////////////
//     CString Extension
/////////////////////////////////////////////////////////////////////
/*
--------------------------------------------------------------------------------

There are innumerable MFC functions that need a CString object as an argument or
return a CString object. Often, even if a function needs a char pointer, we still
use a CString to hold the string. Although, the CString class has lots of features,
it does lack a few very useful string manipulation functions. A case insensitive
search funciton is one, a search and replace function is another. Fortunately,
implementing these functionality is not very difficult. The CStringEx class given
below derives from CString and adds a few useful functions. The CStringEx class 
does not add any member variables of its own, so that it can be freely interchanged
with a CString object. 

The constructors in the CStringEx class parallel the constructor in CString and in
fact simply chains into the corresponding CString constructor. Some of the CStringEx
functions use knowledge of the internal structure of the CString object so there
is a small chance that these functions might break if the CString implementation changes. 

The functions provided by CStringEx are quite easy to use and fairly simple to
understand. The Insert() functions inserts a character or a sub-string within a
string. The result is similar to inserting text in an edit control, the string is
expanded to accommodate the sub-string. The Delete() function removes a segment from
the string and shortens it. The Replace() function removes a sub-string and replaces
it with another. Again, the string size is adjusted depending on the size of the 
sub-string that was removed and the size of the sub-string that was inserted. 

The find family of functions, finds a sub-string in the forward or the
reverse direction. The NoCase version of these functions are case insensitive.
The FindReplace() and FindReplaceNoCase() functions searches for a sub-string
and replaces the matching sub-string with another string. The GetField() and
GetDelimitedField() functions find a token in the string. The table below exemplifies
the uses of these functions. 

String			  Function						        	Result

xyz				    Insert(1, a)					    	xayz
xyz			  	  Insert(1, "ab")   						xabyz
xyz				    Insert(4, "ab")		    				xyz ab
Abcde		  	  Delete(2,2)						    	  Abe
Abcde	  		  Replace(2,2,"xyz")				    Abxyze
Abc	  			  Find(b)					    		    1
Abcabc			  Find(b, 2)					    	  4
AbcAbc			  FindNoCase("aBC", 1)			    3
AbcAbc		  	FindReplace("Ab","Xy")				XycXyc
AbcAbc		  	FindReplace("Ab","Xy", FALSE)	XycAbc
AbcAbc			  FindReplaceNoCase("ab","Xy")	XycXyc
AbcAbc		  	FindReplaceNoCase("ab","Xy", FALSE) XycAbc
AbcAbc			  ReverseFind("bc")				    	4
AbcAbc  			ReverseFindNoCase("abc")		 	3
AbcAbc	  		ReverseFindNoCase("abc", 2)		0
AbcAbc		  	GetField( :, 0 )  					AbcAbc
AbcAbc			  GetField( :, 1 )	  				(Blank string)
Abc:Def 			GetField( :, 0 )		  			Abc
Abc:Def	  		GetField( :, 1 )			  		Def
Abc:Def		  	GetField( :, 2 )				  	(Blank string)
big.red.dog   GetFieldReverse( ., 2 )     red
Name: Zafir		GetField("Name: ", 1)		  		Zafir
(415) 555-7777	GetDelimitedField("(", ")", 0 )		415
*/
